# EPICS IOC Startup Script generated by https://github.com/epics-containers/ibek

cd "/epics/ioc"

epicsEnvSet REF_OBJECT_NAME Ref1

dbLoadDatabase dbd/ioc.dbd
ioc_registerRecordDeviceDriver pdbbase

# This line should appear once only in the pre_init section
# TestValues testValue
TestValues Ref1.127.0.0.1
# this is a comment
# that spans multiple lines
# 
testPreInit "ioc_name={{ ioc_name }}" "'AllObject One String'"
my_str =                    AllObject One String
my_inferred_enum =          third
clock_rate =                dummy
my_mixed_enum_no_default =  
.

dbLoadRecords $(RUNTIME_DIR)/ioc.db
iocInit


testPostInit "ioc_name={{ ioc_name }}" test_value:
this should appear once only in the post_init section
# post init comment

